package org.openxma.dsl.reference.dao;

import org.openxma.dsl.platform.dao.*;
import java.util.Collection;
import java.util.Date;
import org.openxma.dsl.reference.base.ContactInfo;
import org.openxma.dsl.reference.dao.CustomerDao;
import org.openxma.dsl.reference.dto.CustomerAddressDto;
import org.openxma.dsl.reference.dto.CustomerContactReport;
import org.openxma.dsl.reference.dto.CustomerIdDto;
import org.openxma.dsl.reference.dto.CustomerOrderReportDto;
import org.openxma.dsl.reference.dto.CustomerView;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.types.Gender;

/**
 * Represents a Data Access Object, as defined in the
 * Core J2EE Pattern Catalog (see http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html),
 * to abstract and encapsulate all access to entity CustomerDao.
 * @see CustomerDao
 */
public interface CustomerDaoGen extends GenericDao<Customer,String>, EntityFactory<Customer> {
	/**
	 * Sort order constants
	 */
    static final SortOrderSpecification LAST_NAME_ASC = SortOrderSpecification.create("Customer.LastNameAsc").asc("lastName");
	/**
	 * Query name constants
	 */
    static final String GENERATE_CUSTOMER_ORDER_REPORT = "Customer.GenerateCustomerOrderReport";
    static final String LOAD_CUSTOMER_ADDRESS_DTO = "Customer.LoadCustomerAddressDto";
    static final String LOAD_CUSTOMER_ADDRESS_DTO_JOIN_BILLING_ADDRESS = "Customer.LoadCustomerAddressDtoJoinBillingAddress";
    static final String FIND_ALL_BY_FIRST_NAME_AND_LAST_NAME0 = "Customer.FindAllByFirstNameAndLastName0";
    static final String FIND_ALL_BY_FIRST_NAME_AND_LAST_NAME1 = "Customer.FindAllByFirstNameAndLastName1";
    static final String FIND_ALL_LIKE_NAME_JOIN_ORDERS = "Customer.FindAllLikeNameJoinOrders";
    static final String FIND_BY_NAMED_QUERY_CUSTOMER_LAST_NAME = "Customer.FindByNamedQueryCustomerLastName";
    static final String FIND_ALL_LIKE_LAST_NAME_PAGEABLE = "Customer.FindAllLikeLastNamePageable";
    static final String FIND_ALL_IN_NAME = "Customer.FindAllInName";
    static final String FIND_BY_BIRTH_DATE = "Customer.FindByBirthDate";
    static final String LOAD_CUSTOMER_CONTACT_REPORT = "Customer.LoadCustomerContactReport";
    static final String LOAD_UNIQUE_CUSTOMER_CONTACT_REPORT = "Customer.LoadUniqueCustomerContactReport";
    static final String FIND_ALL_CUSTOMER_IDS = "Customer.FindAllCustomerIds";
    static final String FIND_ALL_CUSTOMER_ID_LIKE_NAME = "Customer.FindAllCustomerIdLikeName";
    static final String FIND_CUSTOMER_BY_CONTACT = "Customer.FindCustomerByContact";
    static final String FIND_BY_CUSTOMER_VIEW = "Customer.FindByCustomerView";
    static final String FIND_CUSTOMER_BY_EXAMPLE = "Customer.FindCustomerByExample";
    static final String DELETE_CUSTOMER_BY_NAME = "Customer.DeleteCustomerByName";
    static final String UPDATE_CUSTOMER = "Customer.UpdateCustomer";
    /**
	 * Stored procedure name constants
	 */
    /**
     * Factory method to create an empty <code>Customer</code> entity with an already assigned identifier.
     * The advantage of this approach, coupled with the fact that we're using the id for equals,
     * is that equality test's are immediately possible and therefore independent from the persistence state
     * of the entity. This in contrast to other solutions where auto generated Keys are used
     * which must wait until the object is persisted or differentiate between transient and managed states.
     * @return a new (transient), empty <code>Customer</code> entity with an already assigned identifier.
     */
    Customer create();
    /**
     * Factory method to create an empty <code>Customer</code> taking all not null attributes.
     *
	 * @param invoiceAddress
	 * @param deliveryAddress
	 * @param firstName
	 * @param lastName
	 * @param gender
	 * @param emailAddress
	 * @param birthDate
     * @return a new (transient ) <code>Customer</code> entity with the given natural identifier and not null attributes set.
     */
    Customer create(Address invoiceAddress,Address deliveryAddress,String firstName,String lastName,Gender gender,String emailAddress,Date birthDate);
    /**
     * Factory method to create and save an empty <code>Customer</code> taking all not null attributes.
     *
	 * @param invoiceAddress
	 * @param deliveryAddress
	 * @param firstName
	 * @param lastName
	 * @param gender
	 * @param emailAddress
	 * @param birthDate
     * @return a new (transient) <code>Customer</code> entity with the given natural identifier and not null attributes set.
     */
    Customer createAndSave(Address invoiceAddress,Address deliveryAddress,String firstName,String lastName,Gender gender,String emailAddress,Date birthDate);
    /**
     * Return the persistent instance of the Customer entity class
     * with the given natural identifier or <code>null</code> if not found.
     *
	 * @param firstName
	 * @param lastName
	 * @param birthDate
     * @return the persistent Customer instance, or <code>null</code> if not found
     */
     Customer loadByCustomerNk(String firstName,String lastName,Date birthDate);
    /**
     * Return the persistent instance of the Customer entity class
     * with the given unique-key or <code>null</code> if not found.
     *
	 * @param emailAddress
     * @return the persistent Customer instance, or <code>null</code> if not found
     */
     Customer loadByCustomerUkEmail(String emailAddress);
    /**
     *
     * @return a collection containing 0 or more persistent CustomerOrderReportDto instances
     */
    Collection<CustomerOrderReportDto> generateCustomerOrderReport();
    /**
     *
     * @return a collection containing 0 or more persistent CustomerAddressDto instances
     */
    Collection<CustomerAddressDto> loadCustomerAddressDto();
    /**
     *
     * @return a collection containing 0 or more persistent CustomerAddressDto instances
     */
    Collection<CustomerAddressDto> loadCustomerAddressDtoJoinBillingAddress();
    /**
     *
     * @param firstName
     * @param lastName
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findAllByFirstNameAndLastName(String firstName,String lastName);
    /**
     *
     * @param firstName
     * @param lastName
	 * @param emailAddress
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findAllByFirstNameAndLastName(String firstName,String lastName,String emailAddress);
    /**
     *
     * @param firstName
     * @param lastName
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findAllLikeNameJoinOrders(String firstName,String lastName);
    /**
     *
     * @param lastName
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findByNamedQueryCustomerLastName(String lastName);
    /**
     *
     * @param lastName
     * @param firstResult
     * @param maxResults
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findAllLikeLastNamePageable(String lastName,Integer firstResult,Integer maxResults);
    /**
     *
     * @param lastName
     * @param firstResult
     * @param maxResults
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findAllInName(Collection<String> lastName,Integer firstResult,Integer maxResults);
    /**
     *
     * @param birthDate
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findByBirthDate(Date birthDate);
    /**
     *
     * @return a collection containing 0 or more persistent CustomerContactReport instances
     */
    Collection<CustomerContactReport> loadCustomerContactReport();
    /**
     *
	 * @param lastName
     * @return the customerContactReport instance, or <code>null</code> if not found
     */
    CustomerContactReport loadUniqueCustomerContactReport(String lastName);
    /**
     *
     * @return a collection containing 0 or more persistent CustomerIdDto instances
     */
    Collection<CustomerIdDto> findAllCustomerIds();
    /**
     *
     * @param name
     * @return a collection containing 0 or more persistent String instances
     */
    Collection<String> findAllCustomerIdLikeName(String name);
    /**
     *
     * @param contactInfo
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findCustomerByContact(ContactInfo contactInfo);
    /**
     *
     * @param customerView
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findByCustomerView(CustomerView customerView);
    /**
     *
     * @param customer
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findCustomerByExample(Customer customer);
    /**
     *
	 * @param lastName
     * @return the  instance, or <code>null</code> if not found
     */
    void deleteCustomerByName(String lastName);
    /**
     *
	 * @param lastName
     * @param newLastName
     * @return the  instance, or <code>null</code> if not found
     */
    void updateCustomer(String lastName,String newLastName);
}
